@@ -84,37 +84,6 @@
 #include <event2/bufferevent_struct.h>
 #include <event2/listener.h>
 
-/* Terminology used in this file:
-
-   A "connection" is a bidirectional communications channel, usually
-   backed by a network socket, and represented in this layer by a
-   'conn_t', wrapping a 'struct bufferevent'.
-
-   A "circuit" is a _pair_ of connections, referred to as the
-   "upstream" and "downstream" connections.  A circuit is represented
-   by a 'circuit_t'.  The upstream connection of a circuit
-   communicates in cleartext with the higher-level program that wishes
-   to make use of our obfuscation service.  The downstream connection
-   commmunicates in an obfuscated fashion with the remote peer that
-   the higher-level client wishes to contact.
-
-   A "listener" is a listening socket bound to a particular
-   obfuscation protocol, represented in this layer by a 'listener_t'
-   and its 'config_t'.  Connecting to a listener creates one
-   connection of a circuit, and causes this program to initiate the
-   other connection (possibly after receiving in-band instructions
-   about where to connect to).  A listener is said to be a "client"
-   listener if connecting to it creates the _upstream_ connection, and
-   a "server" listener if connecting to it creates the _downstream_
-   connection.
-
-   There are two kinds of client listeners: a "simple" client listener
-   always connects to the same remote peer every time it needs to
-   initiate a downstream connection; a "socks" client listener can be
-   told to connect to an arbitrary remote peer using the SOCKS protocol
-   (version 4 or 5).
-*/
-
 /**
   This struct defines the state of a listener on a particular address.
  */
@@ -150,7 +119,6 @@ static void conn_free_on_flush(struct bufferevent *bev, void *arg);
 static void circuit_create_socks(conn_t *up);
 static int circuit_add_down(circuit_t *circuit, conn_t *down);
 static void circuit_free(circuit_t *circuit);
-static conn_t *open_outbound(conn_t *conn, bufferevent_data_cb readcb);
 
 static void upstream_read_cb(struct bufferevent *bev, void *arg);
 static void downstream_read_cb(struct bufferevent *bev, void *arg);
@@ -331,6 +299,88 @@ listener_cb(struct evconnlistener *evcl, evutil_socket_t fd,
   }
 }
 
+/** Given a connection <b>addr_conn</b> carrying addresses that we
+    should connect to, use <b>connect_conn</b> to connect to
+    them. Also, set the callbacks of the connecting bufferevent, with
+    <b>readcb</b> being the read callback. */
+static int
+conn_connect(conn_t *addr_conn, conn_t *connect_conn,
+             bufferevent_data_cb readcb)
+{
+  char *peername;
+  struct evutil_addrinfo *addr = config_get_target_addr(addr_conn->cfg);
+  struct bufferevent *buf = connect_conn->buffer;
+
+  if (!addr) {
+    log_warn("%s: no target addresses available", safe_str(addr_conn->peername));
+    return -1;
+  }
+
+  bufferevent_setcb(buf, readcb, NULL, pending_conn_cb, connect_conn);
+
+  do {
+    peername = printable_address(addr->ai_addr, addr->ai_addrlen);
+
+    if (bufferevent_socket_connect(buf, addr->ai_addr, addr->ai_addrlen) >= 0)
+      goto success;
+    log_info("%s: connection to %s failed: %s",
+             safe_str(addr_conn->peername), safe_str(peername),
+             evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR()));
+    free(peername);
+    addr = addr->ai_next;
+  } while (addr);
+
+  log_warn("%s: all outbound connection attempts failed",
+           addr_conn->peername);
+
+  return -1;
+
+ success:
+  log_info("%s (%s): Successful outbound connection to '%s'.",
+           safe_str(addr_conn->peername),
+           addr_conn->cfg->vtable->name, safe_str(peername));
+  bufferevent_enable(buf, EV_READ|EV_WRITE);
+  connect_conn->peername = peername;
+
+  /* If we are a server, we should note this connection and consider
+     it in our heartbeat status messages. */
+  if (addr_conn->mode == LSN_SIMPLE_SERVER)
+    status_note_connection(addr_conn->peername);
+
+  return 0;
+}
+
+/** Create the other side of connection <b>conn</b> on a circuit. */
+static conn_t *
+create_other_side_of_conn(const conn_t *conn)
+{
+  struct event_base *base = bufferevent_get_base(conn->buffer);
+  struct bufferevent *buf;
+  conn_t *newconn;
+
+  buf = bufferevent_socket_new(base, -1, BEV_OPT_CLOSE_ON_FREE);
+  if (!buf) {
+    log_warn("%s: unable to create outbound socket buffer", safe_str(conn->peername));
+    return NULL;
+  }
+
+  newconn = proto_conn_create(conn->cfg);
+  if (!newconn) {
+    log_warn("%s: failed to allocate state for outbound connection",
+             safe_str(conn->peername));
+    bufferevent_free(buf);
+    return NULL;
+  }
+
+  /* associate bufferevent with the new connection. */
+  newconn->buffer = buf;
+
+  obfs_assert(connections);
+  smartlist_add(connections, newconn);
+
+  return newconn;
+}
+
 /**
    This function is called when an upstream client connects to us in
    simple client mode.
@@ -344,11 +394,22 @@ simple_client_listener_cb(conn_t *conn)
 
   bufferevent_setcb(conn->buffer, upstream_read_cb, NULL, error_cb, conn);
 
-  if (circuit_create(conn, open_outbound(conn, downstream_read_cb))) {
+  conn_t *newconn = create_other_side_of_conn(conn);
+  if (!newconn) {
     conn_free(conn);
     return;
   }
 
+  if (circuit_create(conn, newconn)) {
+    conn_free(conn);
+    return;
+  }
+
+  if (conn_connect(conn, newconn, downstream_read_cb) < 0) {
+    conn_free(conn); /* circuit_free() should also free newconn. */
+    return;
+  }
+
   log_debug("%s: setup complete", safe_str(conn->peername));
 }
 
@@ -387,7 +448,18 @@ simple_server_listener_cb(conn_t *conn)
 
   bufferevent_setcb(conn->buffer, downstream_read_cb, NULL, error_cb, conn);
 
-  if (circuit_create(open_outbound(conn, upstream_read_cb), conn)) {
+  conn_t *newconn = create_other_side_of_conn(conn);
+  if (!newconn) {
+    conn_free(conn);
+    return;
+  }
+
+  if (circuit_create(newconn, conn)) {
+    conn_free(conn);
+    return;
+  }
+
+  if (conn_connect(conn, newconn, upstream_read_cb) < 0) {
     conn_free(conn);
     return;
   }
@@ -497,135 +569,6 @@ circuit_free(circuit_t *circuit)
   proto_circuit_free(circuit, cfg);
 }
 
-/**
-   Make the outbound connection for a circuit.
-*/
-static conn_t *
-open_outbound(conn_t *conn, bufferevent_data_cb readcb)
-{
-  struct evutil_addrinfo *addr = config_get_target_addr(conn->cfg);
-  struct event_base *base = bufferevent_get_base(conn->buffer);
-  struct bufferevent *buf;
-  char *peername;
-  conn_t *newconn;
-
-  if (!addr) {
-    log_warn("%s: no target addresses available", safe_str(conn->peername));
-    return NULL;
-  }
-
-  buf = bufferevent_socket_new(base, -1, BEV_OPT_CLOSE_ON_FREE);
-  if (!buf) {
-    log_warn("%s: unable to create outbound socket buffer", safe_str(conn->peername));
-    return NULL;
-  }
-
-  newconn = proto_conn_create(conn->cfg);
-  if (!newconn) {
-    log_warn("%s: failed to allocate state for outbound connection",
-             safe_str(conn->peername));
-    bufferevent_free(buf);
-    return NULL;
-  }
-
-  newconn->buffer = buf;
-  bufferevent_setcb(buf, readcb, NULL, pending_conn_cb, newconn);
-
-  do {
-    peername = printable_address(addr->ai_addr, addr->ai_addrlen);
-    if (bufferevent_socket_connect(buf, addr->ai_addr, addr->ai_addrlen) >= 0)
-      goto success;
-    log_info("%s: connection to %s failed: %s",
-             safe_str(conn->peername), safe_str(peername),
-             evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR()));
-    free(peername);
-    addr = addr->ai_next;
-  } while (addr);
-
-  log_warn("%s: all outbound connection attempts failed",
-           conn->peername);
-
-  conn_free(newconn);
-  return NULL;
-
- success:
-  log_info("%s (%s): Successful outbound connection to '%s'.",
-           safe_str(conn->peername), conn->cfg->vtable->name, safe_str(peername));
-  bufferevent_enable(buf, EV_READ|EV_WRITE);
-  newconn->peername = peername;
-  obfs_assert(connections);
-  smartlist_add(connections, newconn);
-
-  /* If we are a server, we should note this connection and consider
-     it in our heartbeat status messages. */
-  if (conn->mode == LSN_SIMPLE_SERVER)
-    status_note_connection(conn->peername);
-
-  return newconn;
-}
-
-/**
-   As open_outbound, but uses bufferevent_socket_connect_hostname
-   rather than bufferevent_socket_connect.
-*/
-static conn_t *
-open_outbound_hostname(conn_t *conn, int af, const char *addr, uint16_t port)
-{
-  struct event_base *base = bufferevent_get_base(conn->buffer);
-  struct bufferevent *buf;
-  conn_t *newconn;
-
-  buf = bufferevent_socket_new(base, -1, BEV_OPT_CLOSE_ON_FREE);
-  if (!buf) {
-    log_warn("%s: unable to create outbound socket buffer", safe_str(conn->peername));
-    return NULL;
-  }
-  newconn = proto_conn_create(conn->cfg);
-  if (!newconn) {
-    log_warn("%s: failed to allocate state for outbound connection",
-             safe_str(conn->peername));
-    bufferevent_free(buf);
-    return NULL;
-  }
-
-  /* associate bufferevent with the new connection. */
-  newconn->buffer = buf;
-
-  { /* set up temporary FQDN peername. When we connect to the host, we
-        will replace this peername with the IP address we connected
-        to. */
-
-    /* '6' is strlen(":65535")
-       '1' is the ending NUL. */
-    const size_t peername_len = strlen(addr) + 6 + 1;
-    newconn->peername = xzalloc(peername_len);
-    if (obfs_snprintf(newconn->peername, peername_len, "%s:%d",
-                      addr, port) < 0) {
-      log_warn("Failed while writing peername (%s:%u)! This might be a bug.",
-               addr, port);
-      free(newconn->peername);
-      newconn->peername = NULL;
-      conn_free(newconn);
-      return NULL;
-    }
-  }
-
-  bufferevent_setcb(buf, downstream_read_cb, NULL, pending_socks_cb, newconn);
-  if (bufferevent_socket_connect_hostname(buf, get_evdns_base(),
-                                          af, addr, port) < 0) {
-    log_warn("%s: outbound connection to %s:%u failed: %s",
-             safe_str(conn->peername), addr, port,
-             evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR()));
-    conn_free(newconn);
-    return NULL;
-  }
-
-  bufferevent_enable(buf, EV_READ|EV_WRITE);
-  obfs_assert(connections);
-  smartlist_add(connections, newconn);
-  return newconn;
-}
-
 /**
     This callback is responsible for handling SOCKS traffic.
 */
@@ -656,15 +599,37 @@ socks_read_cb(struct bufferevent *bev, void *arg)
 
       log_info("%s: socks: trying to connect to %s:%u",
                safe_str(conn->peername), safe_str(addr), port);
-      if (circuit_add_down(conn->circuit,
-                           open_outbound_hostname(conn, af, addr, port))) {
+      conn_t *newconn = create_other_side_of_conn(conn);
+      if (!newconn) {
+        conn_free(conn);
+        return;
+      }
+
+      /* Set up a temporary FQDN peername. When we actually connect to the
+         host, we will replace this peername with the IP address we
+         connected to. */
+      obfs_asprintf(&newconn->peername, "%s:%u", addr, port);
+
+      if (circuit_add_down(conn->circuit, newconn)) {
         /* XXXX send socks reply */
         conn_free(conn);
         return;
       }
+
+      bufferevent_setcb(newconn->buffer, downstream_read_cb, NULL, pending_socks_cb, newconn);
+
       /* further upstream data will be processed once the downstream
          side is established */
       bufferevent_disable(conn->buffer, EV_READ|EV_WRITE);
+
+      if (bufferevent_socket_connect_hostname(newconn->buffer, get_evdns_base(),
+                                              af, addr, port) < 0) {
+        log_warn("%s: outbound connection to %s:%u failed: %s",
+                 safe_str(conn->peername), addr, port,
+                 evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR()));
+        conn_free(conn);
+        return;
+      }
       return;
     }
 
