@@ -11,6 +11,7 @@ extern struct testcase_t socks_tests[];
 extern struct testcase_t dummy_tests[];
 extern struct testcase_t obfs2_tests[];
 extern struct testcase_t managed_tests[];
+extern struct testcase_t util_tests[];
 
 struct testgroup_t groups[] = {
   { "container/", container_tests },
@@ -19,6 +20,7 @@ struct testgroup_t groups[] = {
   { "dummy/", dummy_tests },
   { "obfs2/", obfs2_tests },
   { "managed/", managed_tests },
+  { "util/", util_tests },
   END_OF_GROUPS
 };
 
